@@ -17,11 +17,11 @@
 
 ;; XEmacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with XEmacs; see the file COPYING.  If not, write to the 
+;; along with XEmacs; see the file COPYING.  If not, write to the
 ;; Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 ;; Boston, MA 02111-1307, USA.
 
@@ -52,6 +52,7 @@
 ;; browse-url-default-windows-browser MS-Windows browser
 ;; browse-url-gnome-moz               GNOME interface to Mozilla
 ;; browse-url-kde                     KDE konqueror (kfm)
+;; browse-url-opera                   Opera
 
 ;; [A version of the Netscape browser is now free software
 ;; <URL:http://www.mozilla.org/>, albeit not GPLed, so it is
@@ -534,7 +535,7 @@
 ;;1999-11-14  Matt Curtin <cmcurtin@interhack.net>
 ;;      * Merge from GNU Emacs 20.4's browse-url.el with XEmacs.
 ;;      Changes from FSF are for sensibility in XEmacs:
-;;      Replaced use of temporary-file-directory with call to 
+;;      Replaced use of temporary-file-directory with call to
 ;;      `temp-directory', change requires considering differences in
 ;;      autoloading, use gnuclient instead of gnudoit.
 ;;1999-12-17 Neal Becker <nbecker@fred.net>
@@ -546,7 +547,9 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Variables
 
-(eval-when-compile (require 'term))
+(eval-when-compile (require 'thingatpt)
+                   (require 'term)
+                   (require 'executable))
 
 (defgroup browse-url nil
   "Use a web browser to look at a URL."
@@ -585,6 +588,7 @@ regexp should probably be \".\" to specify a default browser."
 	  (function-item :tag "Grail" :value  browse-url-grail)
 	  (function-item :tag "MMM" :value  browse-url-mmm)
 	  (function-item :tag "KDE" :value browse-url-kde)
+	  (function-item :tag "Opera" :value browse-url-opera) ;; XEmacs
 	  (function-item :tag "Specified by `Browse Url Generic Program'"
 			 :value browse-url-generic)
 	  (function-item :tag "Default Windows browser"
@@ -594,13 +598,17 @@ regexp should probably be \".\" to specify a default browser."
 	  (function-item :tag "Default browser"
 			 :value browse-url-default-browser)
 	  (function :tag "Your own function")
-	  (alist :tag "Regexp/function association list"
-		 :key-type regexp :value-type function))
+	  ;; XEmacs: Emacs has an `alist' custom type here, but custom
+	  ;; in XEmacs is buggy and cannot handle it, so we don't
+	  ;; include the following:
+	  ;; (alist :tag "Regexp/function association list"
+	  ;;        :key-type regexp :value-type function))
+	  )
   :group 'browse-url)
 
 (defcustom browse-url-netscape-program "netscape"
   ;; Info about netscape-remote from Karl Berry.
-  "The name by which to invoke Netscape.
+  "*The name by which to invoke Netscape.
 
 The free program `netscape-remote' from
 <URL:http://home.netscape.com/newsref/std/remote.c> is said to start
@@ -611,12 +619,12 @@ system, given vroot.h from the same directory, with cc flags
   :group 'browse-url)
 
 (defcustom browse-url-netscape-arguments nil
-  "A list of strings to pass to Netscape as arguments."
+  "*A list of strings to pass to Netscape as arguments."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
 (defcustom browse-url-netscape-startup-arguments browse-url-netscape-arguments
-  "A list of strings to pass to Netscape when it starts up.
+  "*A list of strings to pass to Netscape when it starts up.
 Defaults to the value of `browse-url-netscape-arguments' at the time
 `browse-url' is loaded."
   :type '(repeat (string :tag "Argument"))
@@ -629,17 +637,17 @@ Defaults to the value of `browse-url-netscape-arguments' at the time
   :group 'browse-url)
 
 (defcustom browse-url-mozilla-program "mozilla"
-  "The name by which to invoke Mozilla."
+  "*The name by which to invoke Mozilla."
   :type 'string
   :group 'browse-url)
 
 (defcustom browse-url-mozilla-arguments nil
-  "A list of strings to pass to Mozilla as arguments."
+  "*A list of strings to pass to Mozilla as arguments."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
 (defcustom browse-url-mozilla-startup-arguments browse-url-mozilla-arguments
-  "A list of strings to pass to Mozilla when it starts up.
+  "*A list of strings to pass to Mozilla when it starts up.
 Defaults to the value of `browse-url-mozilla-arguments' at the time
 `browse-url' is loaded."
   :type '(repeat (string :tag "Argument"))
@@ -647,24 +655,24 @@ Defaults to the value of `browse-url-mozilla-arguments' at the time
 
 ;;;###autoload
 (defcustom browse-url-galeon-program "galeon"
-  "The name by which to invoke Galeon."
+  "*The name by which to invoke Galeon."
   :type 'string
   :group 'browse-url)
 
 (defcustom browse-url-galeon-arguments nil
-  "A list of strings to pass to Galeon as arguments."
+  "*A list of strings to pass to Galeon as arguments."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
 (defcustom browse-url-galeon-startup-arguments browse-url-galeon-arguments
-  "A list of strings to pass to Galeon when it starts up.
+  "*A list of strings to pass to Galeon when it starts up.
 Defaults to the value of `browse-url-galeon-arguments' at the time
 `browse-url' is loaded."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
 (defcustom browse-url-galeon-new-window-is-tab nil
-  "Whether to open up new windows in a tab or a new window.
+  "*Whether to open up new windows in a tab or a new window.
 If non-nil, then open the URL in a new tab rather than a new window if
 `browse-url-galeon' is asked to open it in a new window."
   :type 'boolean
@@ -684,17 +692,17 @@ commands reverses the effect of this variable.  Requires Netscape version
 (defvaralias 'browse-url-new-window-p 'browse-url-new-window-flag)
 
 (defcustom browse-url-mosaic-program "xmosaic"
-  "The name by which to invoke Mosaic (or mMosaic)."
+  "*The name by which to invoke Mosaic (or mMosaic)."
   :type 'string
   :group 'browse-url)
 
 (defcustom browse-url-mosaic-arguments nil
-  "A list of strings to pass to Mosaic as arguments."
+  "*A list of strings to pass to Mosaic as arguments."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
 (defcustom browse-url-mosaic-pidfile "~/.mosaicpid"
-  "The name of the pidfile created by Mosaic."
+  "*The name of the pidfile created by Mosaic."
   :type 'string
   :group 'browse-url)
 
@@ -739,7 +747,7 @@ Used by the `browse-url-of-file' command."
   :group 'browse-url)
 
 (defcustom browse-url-of-file-hook nil
-  "Run after `browse-url-of-file' has asked a browser to load a file.
+  "*Run after `browse-url-of-file' has asked a browser to load a file.
 
 Set this to `browse-url-netscape-reload' to force Netscape to load the
 file rather than displaying a cached copy."
@@ -757,7 +765,7 @@ Emacs 19.29 accepts 'SIGUSR1, earlier versions require an integer
 which is 30 on SunOS and 16 on HP-UX and Solaris.")
 
 (defcustom browse-url-CCI-port 3003
-  "Port to access XMosaic via CCI.
+  "*Port to access XMosaic via CCI.
 This can be any number between 1024 and 65535 but must correspond to
 the value set in the browser."
   :type 'integer
@@ -774,7 +782,7 @@ enabled.  The port number should be set in `browse-url-CCI-port'."
 (make-variable-buffer-local 'browse-url-temp-file-name)
 
 (defcustom browse-url-xterm-program "xterm"
-  "The name of the terminal emulator used by `browse-url-lynx-xterm'.
+  "*The name of the terminal emulator used by `browse-url-lynx-xterm'.
 This might, for instance, be a separate colour version of xterm."
   :type 'string
   :group 'browse-url)
@@ -787,7 +795,7 @@ These might set its size, for instance."
 
 (defcustom browse-url-lynx-emacs-args (and (not window-system)
                                            '("-show_cursor"))
-  "A list of strings defining options for Lynx in an Emacs buffer.
+  "*A list of strings defining options for Lynx in an Emacs buffer.
 
 The default is none in a window system, otherwise `-show_cursor' to
 indicate the position of the current link in the absence of
@@ -795,12 +803,12 @@ highlighting, assuming the normal default for showing the cursor."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
-(defcustom browse-url-gnudoit-program "gnuclient"
-  "The name of the `gnudoit' program used by `browse-url-w3-gnudoit'."
+(defcustom browse-url-gnudoit-program "gnuclient" ;; XEmacs
+  "*The name of the `gnudoit' program used by `browse-url-w3-gnudoit'."
   :type 'string
   :group 'browse-url)
 
-(defcustom browse-url-gnudoit-args '("-batch" "-eval")
+(defcustom browse-url-gnudoit-args '("-batch" "-eval") ;; XEmacs
   "*A list of strings defining options for `browse-url-gnudoit-program'.
 These might set the port, for instance."
   :type '(repeat (string :tag "Argument"))
@@ -817,8 +825,8 @@ These might set the port, for instance."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
-(defcustom browse-url-temp-dir (temp-directory)
-  "The name of a directory for browse-url's temporary files.
+(defcustom browse-url-temp-dir (temp-directory) ;; XEmacs change
+  "*The name of a directory for browse-url's temporary files.
 Such files are generated by functions like `browse-url-of-region'.
 You might want to set this to somewhere with restricted read permissions
 for privacy's sake."
@@ -827,7 +835,7 @@ for privacy's sake."
 
 (defcustom browse-url-netscape-version
   3
-  "The version of Netscape you are using.
+  "*The version of Netscape you are using.
 This affects how URL reloading is done; the mechanism changed
 incompatibly at version 4."
   :type 'number
@@ -846,16 +854,18 @@ down (this *won't* always work)."
                  (const :tag "Warn, don't emit URL" :value warn))
   :group 'browse-url)
 
-(defvar browse-url-lynx-input-attempts 10
-  "*How many times to try to move down from a series of lynx input fields.")
+(defcustom browse-url-lynx-input-attempts 10
+  "*How many times to try to move down from a series of lynx input fields."
+:type 'integer
+:group 'browse-url)
 
 (defcustom browse-url-lynx-input-delay 0.2
-  "How many seconds to wait for lynx between moves down from an input field.")
-
-(defvar browse-url-temp-file-list '())
+  "*How many seconds to wait for lynx between moves down from an input field."
+:type 'number
+:group 'browse-url)
 
 (defcustom browse-url-kde-program "kfmclient"
-  "The name by which to invoke the KDE web browser."
+  "*The name by which to invoke the KDE web browser."
   :type 'string
   :group 'browse-url)
 
@@ -864,6 +874,23 @@ down (this *won't* always work)."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
+(defcustom browse-url-opera-program "opera"
+  "*The name by which to invoke Opera."
+  :type 'string
+  :group 'browse-url)
+
+(defcustom browse-url-opera-arguments '()
+  "*A list of strings to pass to Opera as arguments."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
+(defcustom browse-url-opera-startup-arguments browse-url-opera-arguments
+  "*A list of strings to pass to Opera when it starts up.
+Defaults to the value of `browse-url-opera-arguments' at the time
+`browse-url' is loaded."
+  :type '(repeat (string :tag "Argument"))
+  :group 'browse-url)
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; URL input
 
@@ -894,7 +921,7 @@ negation if a prefix argument was given."
 ;; interactive-p needs to be called at a function's top-level, hence
 ;; the macro.
 (defmacro browse-url-maybe-new-window (arg)
-  `(if (interactive-p)
+  `(if (not (interactive-p))
        ,arg
      browse-url-new-window-flag))
 
@@ -963,31 +990,14 @@ narrowed."
 	    (write-region (point-min) (point-max) file-name nil 'no-message)))
       (browse-url-of-file file-name))))
 
-;; XEmacs modifications
 (defun browse-url-delete-temp-file (&optional temp-file-name)
-  ;; Delete browse-url-temp-file-name from the file system and from
-  ;; browse-url-temp-file-list.  If optional arg TEMP-FILE-NAME is
-  ;; non-nil, delete it instead, but only from the file system --
-  ;; browse-url-temp-file-list is not affected.
+  "Delete `browse-url-temp-file-name' from the file system.
+If the optional argument TEMP-FILE-NAME is non-nil, delete it instead."
   (let ((file-name (or temp-file-name browse-url-temp-file-name)))
     (if (and file-name (file-exists-p file-name))
-	(progn
-	  (delete-file file-name)
-	  (if (null temp-file-name)
-	      (setq browse-url-temp-file-list
-		    (delete browse-url-temp-file-name
-			    browse-url-temp-file-list)))))))
-
-;; XEmacs modifications
-(defun browse-url-delete-temp-file-list ()
-  ;; Delete all elements of browse-url-temp-file-list.
-  (while browse-url-temp-file-list
-    (browse-url-delete-temp-file (car browse-url-temp-file-list))
-    (setq browse-url-temp-file-list
-	  (cdr browse-url-temp-file-list))))
+	(delete-file file-name))))
 
 (add-hook 'kill-buffer-hook 'browse-url-delete-temp-file)
-(add-hook 'kill-emacs-hook 'browse-url-delete-temp-file-list)
 
 ;;;###autoload
 (defun browse-url-of-dired-file ()
@@ -1052,7 +1062,9 @@ to use."
   (interactive "e")
   (save-excursion
     (mouse-set-point event)
-    (browse-url-at-point browse-url-new-window-flag)))
+    ;; This handles browse-url-new-window-flag properly
+    ;; when it gets no arg.
+    (browse-url-at-point)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Browser-specific commands
@@ -1100,8 +1112,8 @@ the effect of `browse-url-new-window-flag'.
 When called non-interactively, optional second argument NEW-WINDOW is
 used instead of `browse-url-new-window-flag'.
 
-The order attempted is gnome-moz-remote, Mozilla, Galeon, Netscape,
-Mosaic, IXI Mosaic, Lynx in an xterm, MMM, Konqueror, and then W3."
+The order attempted is gnome-moz-remote, Mozilla, Galeon, Konqueror,
+Netscape, Opera, Mosaic, IXI Mosaic, Lynx in an xterm, MMM, and then W3."
   (apply
     (cond
      ((executable-find "gnome-moz-remote") 'browse-url-gnome-moz)
@@ -1109,6 +1121,7 @@ Mosaic, IXI Mosaic, Lynx in an xterm, MMM, Konqueror, and then W3."
      ((executable-find browse-url-galeon-program) 'browse-url-galeon)
      ((executable-find browse-url-kde-program) 'browse-url-kde)
      ((executable-find browse-url-netscape-program) 'browse-url-netscape)
+     ((executable-find browse-url-opera-program) 'browse-url-opera)
      ((executable-find browse-url-mosaic-program) 'browse-url-mosaic)
      ((executable-find "tellw3b") 'browse-url-iximosaic)
      ((executable-find browse-url-xterm-program) 'browse-url-lynx-xterm)
@@ -1176,7 +1189,7 @@ How depends on `browse-url-netscape-version'."
 				"reload")))
 
 (defun browse-url-netscape-send (command)
-  "Send a remote control command to Netscape."
+  "Send a remote control COMMAND to Netscape."
   (let* ((process-environment (browse-url-process-environment)))
     (apply 'start-process "netscape" nil
            browse-url-netscape-program
@@ -1238,6 +1251,10 @@ non-nil, load the document in a new Galeon window, otherwise use a
 random existing one.  A non-nil interactive prefix argument reverses
 the effect of `browse-url-new-window-flag'.
 
+If `browse-url-galeon-new-window-is-tab' is non-nil, then whenever a
+document would otherwise be loaded in a new window, it is loaded in a
+new tab in an existing window instead.
+
 When called non-interactively, optional second argument NEW-WINDOW is
 used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "URL: "))
@@ -1255,8 +1272,9 @@ used instead of `browse-url-new-window-flag'."
                           (if (browse-url-maybe-new-window new-window)
 			      (if browse-url-galeon-new-window-is-tab
 				  '("--new-tab")
-				'("--new-window" "--no-raise")))
-                          (list "-x" url)))))
+				'("--new-window" "--noraise"))
+                            '("--existing"))
+                          (list url)))))
     (set-process-sentinel process
 			  `(lambda (process change)
 			     (browse-url-galeon-sentinel process ,url)))))
@@ -1274,7 +1292,7 @@ used instead of `browse-url-new-window-flag'."
 ;; GNOME means of invoking either Mozilla or Netrape.
 
 (defcustom browse-url-gnome-moz-arguments '()
-  "A list of strings passed to the GNOME mozilla viewer as arguments."
+  "*A list of strings passed to the GNOME mozilla viewer as arguments."
   :type '(repeat (string :tag "Argument"))
   :group 'browse-url)
 
@@ -1291,7 +1309,7 @@ effect of `browse-url-new-window-flag'.
 
 When called non-interactively, optional second argument NEW-WINDOW is
 used instead of `browse-url-new-window-flag'."
-  (interactive (browse-url-interactive-arg "URL: "))  
+  (interactive (browse-url-interactive-arg "URL: "))
   (apply 'start-process (concat "gnome-moz-remote " url)
 	 nil
 	 "gnome-moz-remote"
@@ -1476,7 +1494,7 @@ non-nil, load the document in a new lynx in a new term window,
 otherwise use any existing one.  A non-nil interactive prefix argument
 reverses the effect of `browse-url-new-window-flag'.
 
-When called non-interactively, optional second argument NEW-WINDOW is
+When called non-interactively, optional second argument NEW-BUFFER is
 used instead of `browse-url-new-window-flag'."
   (interactive (browse-url-interactive-arg "Lynx URL: "))
   (let* ((system-uses-terminfo t)       ; Lynx uses terminfo
@@ -1593,6 +1611,8 @@ don't offer a form of remote control."
 	 browse-url-generic-program
 	 (append browse-url-generic-args (list url))))
 
+;; --- KDE ---
+
 ;;;###autoload
 (defun browse-url-kde (url &optional new-window)
   "Ask the KDE WWW browser to load URL.
@@ -1604,6 +1624,52 @@ Default to the URL around or before point."
 
 (defalias 'browse-url-kfm 'browse-url-kde) ;; XEmacs backwards compatibility
 
+;; --- Opera ---
+
+;;;###autoload
+(defun browse-url-opera (url &optional new-window)
+  "Ask the Opera WWW browser to load URL.
+Default to the URL around or before point.  The strings in variable
+`browse-url-opera-arguments' are also passed to Opera.
+
+When called interactively, if variable `browse-url-new-window-flag' is
+non-nil, load the document in a new Opera window, otherwise use an
+existing one.  A non-nil interactive prefix argument reverses the
+effect of `browse-url-new-window-flag'.
+
+When called non-interactively, optional second argument NEW-WINDOW is
+used instead of `browse-url-new-window-flag'."
+  (interactive (browse-url-interactive-arg "URL: "))
+  ;; URL encode any `confusing' characters in the URL.  This needs to
+  ;; include at least commas; presumably also close parens.
+  (while (string-match "[,)]" url)
+    (setq url (replace-match
+	       (format "%%%x" (string-to-char (match-string 0 url))) t t url)))
+  (let* ((process-environment (browse-url-process-environment))
+         (process (apply 'start-process
+			 (concat "opera " url) nil
+			 browse-url-opera-program
+			 (append
+			  browse-url-opera-arguments
+			  (list "-remote"
+				(concat "openURL("
+					url
+					(if new-window ",new-window")
+					")"))))))
+    (set-process-sentinel process
+			  `(lambda (process change)
+			     (browse-url-opera-sentinel process ,url)))))
+
+(defun browse-url-opera-sentinel (process url)
+  "Handle a change to the process communicating with Opera."
+  (or (eq (process-exit-status process) 0)
+      (let* ((process-environment (browse-url-process-environment)))
+	;; Opera is not running - start it
+	(message "Starting Opera...")
+	(apply 'start-process (concat "opera " url) nil
+	       browse-url-opera-program
+	       (append browse-url-opera-startup-arguments (list url))))))
+
 (provide 'browse-url)
 
 ;;; browse-url.el ends here
